@@ -4,8 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2 of the License, or (at your
- * option) any later version.
+ * Free Software Foundation; only version 2 of the License is applicable.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,13 +23,13 @@
 #include "common.h"
 #include "plugin.h"
 #include "configfile.h"
-#include "utils_debug.h"
-
-#define MODULE_NAME "ping"
 
 #include <netinet/in.h>
 #include "liboping/oping.h"
 
+/*
+ * Private data types
+ */
 struct hostlist_s
 {
 	char *host;
@@ -40,19 +39,13 @@ struct hostlist_s
 };
 typedef struct hostlist_s hostlist_t;
 
+/*
+ * Private variables
+ */
 static pingobj_t *pingobj = NULL;
 static hostlist_t *hosts = NULL;
 
-static char *file_template = "ping-%s.rrd";
-
-static char *ds_def[] = 
-{
-	"DS:ping:GAUGE:"COLLECTD_HEARTBEAT":0:65535",
-	NULL
-};
-static int ds_num = 1;
-
-static char *config_keys[] =
+static const char *config_keys[] =
 {
 	"Host",
 	"TTL",
@@ -60,25 +53,28 @@ static char *config_keys[] =
 };
 static int config_keys_num = 2;
 
+/*
+ * Private functions
+ */
 static void add_hosts (void)
 {
 	hostlist_t *hl_this;
 	hostlist_t *hl_prev;
 
-	int step = atoi (COLLECTD_STEP);
-
 	hl_this = hosts;
 	hl_prev = NULL;
 	while (hl_this != NULL)
 	{
-		DBG ("host = %s, wait_left = %i, wait_time = %i, next = %p",
-				hl_this->host, hl_this->wait_left, hl_this->wait_time, (void *) hl_this->next);
+		DEBUG ("ping plugin: host = %s, wait_left = %i, "
+				"wait_time = %i, next = %p",
+				hl_this->host, hl_this->wait_left,
+				hl_this->wait_time, (void *) hl_this->next);
 
 		if (hl_this->wait_left <= 0)
 		{
 			if (ping_host_add (pingobj, hl_this->host) == 0)
 			{
-				DBG ("Successfully added host %s", hl_this->host);
+				DEBUG ("ping plugin: Successfully added host %s", hl_this->host);
 				/* Remove the host from the linked list */
 				if (hl_prev != NULL)
 					hl_prev->next = hl_this->next;
@@ -90,6 +86,9 @@ static void add_hosts (void)
 			}
 			else
 			{
+				WARNING ("ping plugin: Failed adding host "
+						"`%s': %s", hl_this->host,
+						ping_get_error (pingobj));
 				hl_this->wait_left = hl_this->wait_time;
 				hl_this->wait_time *= 2;
 				if (hl_this->wait_time > 86400)
@@ -98,7 +97,7 @@ static void add_hosts (void)
 		}
 		else
 		{
-			hl_this->wait_left -= step;
+			hl_this->wait_left -= interval_g;
 		}
 
 		if (hl_this != NULL)
@@ -109,20 +108,21 @@ static void add_hosts (void)
 	}
 }
 
-static void ping_init (void)
+static int ping_init (void)
 {
 	if (hosts != NULL)
 		add_hosts ();
+
+	return (0);
 }
 
-static int ping_config (char *key, char *value)
+static int ping_config (const char *key, const char *value)
 {
 	if (pingobj == NULL)
 	{
 		if ((pingobj = ping_construct ()) == NULL)
 		{
-			syslog (LOG_ERR, "ping: `ping_construct' failed: %s",
-				       	ping_get_error (pingobj));
+			ERROR ("ping plugin: `ping_construct' failed.");
 			return (1);
 		}
 	}
@@ -131,24 +131,27 @@ static int ping_config (char *key, char *value)
 	{
 		hostlist_t *hl;
 		char *host;
-		int step = atoi (COLLECTD_STEP);
 
 		if ((hl = (hostlist_t *) malloc (sizeof (hostlist_t))) == NULL)
 		{
-			syslog (LOG_ERR, "ping plugin: malloc failed: %s",
-					strerror (errno));
+			char errbuf[1024];
+			ERROR ("ping plugin: malloc failed: %s",
+					sstrerror (errno, errbuf,
+						sizeof (errbuf)));
 			return (1);
 		}
 		if ((host = strdup (value)) == NULL)
 		{
+			char errbuf[1024];
 			free (hl);
-			syslog (LOG_ERR, "ping plugin: strdup failed: %s",
-					strerror (errno));
+			ERROR ("ping plugin: strdup failed: %s",
+					sstrerror (errno, errbuf,
+						sizeof (errbuf)));
 			return (1);
 		}
 
 		hl->host = host;
-		hl->wait_time = 2 * step;
+		hl->wait_time = 2 * interval_g;
 		hl->wait_left = 0;
 		hl->next = hosts;
 		hosts = hl;
@@ -158,7 +161,7 @@ static int ping_config (char *key, char *value)
 		int ttl = atoi (value);
 		if (ping_setopt (pingobj, PING_DEF_TIMEOUT, (void *) &ttl))
 		{
-			syslog (LOG_WARNING, "ping: liboping did not accept the TTL value %i", ttl);
+			WARNING ("ping: liboping did not accept the TTL value %i", ttl);
 			return (1);
 		}
 	}
@@ -170,53 +173,47 @@ static int ping_config (char *key, char *value)
 	return (0);
 }
 
-static void ping_write (char *host, char *inst, char *val)
-{
-	char file[512];
-	int status;
-
-	status = snprintf (file, 512, file_template, inst);
-	if (status < 1)
-		return;
-	else if (status >= 512)
-		return;
-
-	rrd_update_file (host, file, val, ds_def, ds_num);
-}
-
-#define BUFSIZE 256
 static void ping_submit (char *host, double latency)
 {
-	char buf[BUFSIZE];
+	value_t values[1];
+	value_list_t vl = VALUE_LIST_INIT;
 
-	if (snprintf (buf, BUFSIZE, "%u:%f", (unsigned int) curtime, latency) >= BUFSIZE)
-		return;
+	values[0].gauge = latency;
 
-	plugin_submit (MODULE_NAME, host, buf);
+	vl.values = values;
+	vl.values_len = 1;
+	vl.time = time (NULL);
+	strcpy (vl.host, hostname_g);
+	strcpy (vl.plugin, "ping");
+	strcpy (vl.plugin_instance, "");
+	strncpy (vl.type_instance, host, sizeof (vl.type_instance));
+
+	plugin_dispatch_values ("ping", &vl);
 }
-#undef BUFSIZE
 
-static void ping_read (void)
+static int ping_read (void)
 {
 	pingobj_iter_t *iter;
 
 	char   host[512];
 	double latency;
 	size_t buf_len;
+	int    number_of_hosts;
 
 	if (pingobj == NULL)
-		return;
+		return (-1);
 
 	if (hosts != NULL)
 		add_hosts ();
 
 	if (ping_send (pingobj) < 0)
 	{
-		syslog (LOG_ERR, "ping: `ping_send' failed: %s",
+		ERROR ("ping plugin: `ping_send' failed: %s",
 				ping_get_error (pingobj));
-		return;
+		return (-1);
 	}
 
+	number_of_hosts = 0;
 	for (iter = ping_iterator_get (pingobj);
 			iter != NULL;
 			iter = ping_iterator_next (iter))
@@ -224,22 +221,38 @@ static void ping_read (void)
 		buf_len = sizeof (host);
 		if (ping_iterator_get_info (iter, PING_INFO_HOSTNAME,
 					host, &buf_len))
+		{
+			WARNING ("ping plugin: ping_iterator_get_info "
+					"(PING_INFO_HOSTNAME) failed.");
 			continue;
+		}
 
 		buf_len = sizeof (latency);
 		if (ping_iterator_get_info (iter, PING_INFO_LATENCY,
 					&latency, &buf_len))
+		{
+			WARNING ("ping plugin: ping_iterator_get_info (%s, "
+					"PING_INFO_LATENCY) failed.", host);
 			continue;
+		}
 
-		DBG ("host = %s, latency = %f", host, latency);
+		DEBUG ("ping plugin: host = %s, latency = %f", host, latency);
 		ping_submit (host, latency);
+		number_of_hosts++;
 	}
-}
+
+	if ((number_of_hosts == 0) && (getuid () != 0))
+	{
+		ERROR ("ping plugin: All hosts failed. Try starting collectd as root.");
+	}
+
+	return (number_of_hosts == 0 ? -1 : 0);
+} /* int ping_read */
 
 void module_register (void)
 {
-	plugin_register (MODULE_NAME, ping_init, ping_read, ping_write);
-	cf_register (MODULE_NAME, ping_config, config_keys, config_keys_num);
-}
-
-#undef MODULE_NAME
+	plugin_register_config ("ping", ping_config,
+			config_keys, config_keys_num);
+	plugin_register_init ("ping", ping_init);
+	plugin_register_read ("ping", ping_read);
+} /* void module_register */
