@@ -1,11 +1,10 @@
 /**
  * collectd - src/users.c
- * Copyright (C) 2005,2006  Sebastian Harl
+ * Copyright (C) 2005-2007  Sebastian Harl
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2 of the License, or (at your
- * option) any later version.
+ * Free Software Foundation; only version 2 of the license is applicable.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,60 +25,33 @@
 
 #if HAVE_UTMPX_H
 # include <utmpx.h>
-#else /* !HAVE_UTMPX_H */
-# if HAVE_UTMP_H
-#  include <utmp.h>
-# endif /* HAVE_UTMP_H */
-#endif /* HAVE_UTMPX_H */
+/* #endif HAVE_UTMPX_H */
 
-#define MODULE_NAME "users"
+#elif HAVE_UTMP_H
+# include <utmp.h>
+/* #endif HAVE_UTMP_H */
 
-#if HAVE_GETUTXENT || HAVE_GETUTENT
-# define USERS_HAVE_READ 1
 #else
-# define USERS_HAVE_READ 0
+# error "No applicable input method."
 #endif
 
-static char *rrd_file = "users.rrd";
-static char *ds_def[] =
+static void users_submit (gauge_t value)
 {
-	"DS:users:GAUGE:"COLLECTD_HEARTBEAT":0:65535",
-	NULL
-};
-static int ds_num = 1;
+	value_t values[1];
+	value_list_t vl = VALUE_LIST_INIT;
 
-static void users_init (void)
-{
-	/* we have nothing to do here :-) */
-	return;
-} /* static void users_init(void) */
+	values[0].gauge = value;
 
-static void users_write (char *host, char *inst, char *val)
-{
-	rrd_update_file(host, rrd_file, val, ds_def, ds_num);
-	return;
-} /* static void users_write(char *host, char *inst, char *val) */
+	vl.values = values;
+	vl.values_len = 1;
+	vl.time = time (NULL);
+	strcpy (vl.host, hostname_g);
+	strcpy (vl.plugin, "users");
 
-#if USERS_HAVE_READ
-/* I don't like this temporary macro definition - well it's used everywhere
-   else in the collectd-sources, so I will just stick with it...  */
-#define BUFSIZE 256
-static void users_submit (unsigned int users)
-{
-	char buf[BUFSIZE] = "";
+	plugin_dispatch_values ("users", &vl);
+} /* void users_submit */
 
-	if (snprintf(buf, BUFSIZE, "%u:%u", 
-		(unsigned int)curtime, users) >= BUFSIZE)
-	{
-		return;
-	}
-
-	plugin_submit(MODULE_NAME, "-", buf);
-	return;
-} /* static void users_submit(unsigned int users) */
-#undef BUFSIZE
-
-static void users_read (void)
+static int users_read (void)
 {
 #if HAVE_GETUTXENT
 	unsigned int users = 0;
@@ -114,17 +86,17 @@ static void users_read (void)
 	}
 	endutent();
 
-	users_submit(users);
-#endif /* HAVE_GETUTENT */
+	users_submit (users);
+/* #endif HAVE_GETUTENT */
 
-	return;
-} /* static void users_read(void) */
 #else
-# define users_read NULL
-#endif /* USERS_HAVE_READ */
+# error "No applicable input method."
+#endif
+
+	return (0);
+} /* int users_read */
 
 void module_register (void)
 {
-	plugin_register (MODULE_NAME, users_init, users_read, users_write);
-	return;
+	plugin_register_read ("users", users_read);
 } /* void module_register(void) */
